@@ -1,23 +1,39 @@
 # -*- coding: utf-8 -*-
 #
 # Copyright (C) 2008 John Paulett (john -at- paulett.org)
+# Copyright (C) 2009, 2011, 2013 David Aguilar
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution.
 
-import os
-import sys
-import doctest
-import unittest
+import collections
 import datetime
+import doctest
+import os
 import time
+import unittest
+import sys
 
 import jsonpickle
+
 from jsonpickle import handlers
 from jsonpickle import tags
+from jsonpickle.compat import unicode
+
+from jsonpickle._samples import (
+        BrokenReprThing,
+        DictSubclass,
+        ListSubclass,
+        ListSubclassWithInit,
+        NamedTuple,
+        ObjWithJsonPickleRepr,
+        SetSubclass,
+        Thing,
+        ThingWithSlots,
+        ThingWithProps,
+        )
 
-from samples import Thing, ThingWithSlots, ThingWithProps, BrokenReprThing, DictSubclass, ListSubclass, SetSubclass
 
 class PicklingTestCase(unittest.TestCase):
     def setUp(self):
@@ -29,8 +45,8 @@ class PicklingTestCase(unittest.TestCase):
         self.assertEqual('a string', self.unpickler.restore('a string'))
 
     def test_unicode(self):
-        self.assertEqual(u'a string', self.pickler.flatten(u'a string'))
-        self.assertEqual(u'a string', self.unpickler.restore(u'a string'))
+        self.assertEqual(unicode('a string'), self.pickler.flatten('a string'))
+        self.assertEqual(unicode('a string'), self.unpickler.restore('a string'))
 
     def test_int(self):
         self.assertEqual(3, self.pickler.flatten(3))
@@ -159,6 +175,49 @@ class PicklingTestCase(unittest.TestCase):
         newdata = jsonpickle.decode(jsonpickle.encode(data))
         self.assertEqual(data, newdata)
 
+    def test_defaultdict_roundtrip(self):
+        """Make sure we can handle collections.defaultdict(list)"""
+        # setup
+        defaultdict = collections.defaultdict(list)
+        defaultdict['a'] = 1
+        defaultdict['b'].append(2)
+        defaultdict['c'] = collections.defaultdict(dict)
+        # jsonpickle work your magic
+        encoded = jsonpickle.encode(defaultdict)
+        newdefaultdict = jsonpickle.decode(encoded)
+        # jsonpickle never fails
+        self.assertEqual(newdefaultdict['a'], 1)
+        self.assertEqual(newdefaultdict['b'], [2])
+        self.assertEqual(type(newdefaultdict['c']), collections.defaultdict)
+        self.assertEqual(defaultdict.default_factory, list)
+        self.assertEqual(newdefaultdict.default_factory, list)
+
+    def test_deque_roundtrip(self):
+        """Make sure we can handle collections.deque"""
+        old_deque = collections.deque([0, 1, 2])
+        encoded = jsonpickle.encode(old_deque)
+        new_deque = jsonpickle.decode(encoded)
+        self.assertNotEqual(encoded, 'nil')
+        self.assertEqual(old_deque[0], 0)
+        self.assertEqual(new_deque[0], 0)
+        self.assertEqual(old_deque[1], 1)
+        self.assertEqual(new_deque[1], 1)
+        self.assertEqual(old_deque[2], 2)
+        self.assertEqual(new_deque[2], 2)
+
+    def test_namedtuple_roundtrip(self):
+        old_nt = NamedTuple(0, 1, 2)
+        encoded = jsonpickle.encode(old_nt)
+        new_nt = jsonpickle.decode(encoded)
+        self.assertEqual(type(old_nt), type(new_nt))
+        self.assertTrue(old_nt is not new_nt)
+        self.assertEqual(old_nt.a, new_nt.a)
+        self.assertEqual(old_nt.b, new_nt.b)
+        self.assertEqual(old_nt.c, new_nt.c)
+        self.assertEqual(old_nt[0], new_nt[0])
+        self.assertEqual(old_nt[1], new_nt[1])
+        self.assertEqual(old_nt[2], new_nt[2])
+
     def test_class(self):
         inst = Thing('test name')
         inst.child = Thing('child name')
@@ -274,11 +333,12 @@ class PicklingTestCase(unittest.TestCase):
 
         flattened = self.pickler.flatten(obj)
         self.assertEqual({'key1': 1,
-                          tags.OBJECT: 'samples.DictSubclass'
+                          tags.OBJECT:
+                            'jsonpickle._samples.DictSubclass'
                          },
                          flattened)
         self.assertEqual(flattened[tags.OBJECT],
-                         'samples.DictSubclass')
+                         'jsonpickle._samples.DictSubclass')
 
         inflated = self.unpickler.restore(flattened)
         self.assertEqual(1, inflated['key1'])
@@ -313,12 +373,28 @@ class PicklingTestCase(unittest.TestCase):
         obj = datetime.datetime.now()
 
         flattened = self.pickler.flatten(obj)
-        self.assertTrue(repr(obj) in flattened[tags.REPR])
-        self.assertTrue('datetime' in flattened[tags.REPR])
+        self.assertTrue(tags.OBJECT in flattened)
+        self.assertTrue('__reduce__' in flattened)
 
         inflated = self.unpickler.restore(flattened)
         self.assertEqual(obj, inflated)
 
+    def test_datetime_inside_int_keys_defaults(self):
+        t = datetime.time(hour=10)
+        s = jsonpickle.encode({1:t, 2:t})
+        d = jsonpickle.decode(s)
+        self.assertEqual(d["1"], d["2"])
+        self.assertTrue(d["1"] is d["2"])
+        self.assertTrue(isinstance(d["1"], datetime.time))
+
+    def test_datetime_inside_int_keys_with_keys_enabled(self):
+        t = datetime.time(hour=10)
+        s = jsonpickle.encode({1:t, 2:t}, keys=True)
+        d = jsonpickle.decode(s, keys=True)
+        self.assertEqual(d[1], d[2])
+        self.assertTrue(d[1] is d[2])
+        self.assertTrue(isinstance(d[1], datetime.time))
+
     def test_broken_repr_dict_key(self):
         """Tests that we can pickle dictionaries with keys that have
         broken __repr__ implementations.
@@ -356,36 +432,6 @@ class PicklingTestCase(unittest.TestCase):
         inflated = self.unpickler.restore(flattened)
         self.assertEqual(inflated.submodule, sysconfig)
 
-    def test_datetime_date(self):
-        obj = datetime.datetime.now().date()
-
-        flattened = self.pickler.flatten(obj)
-        self.assertTrue(repr(obj) in flattened[tags.REPR])
-        self.assertTrue('datetime' in flattened[tags.REPR])
-
-        inflated = self.unpickler.restore(flattened)
-        self.assertEqual(obj, inflated)
-
-    def test_datetime_time(self):
-        obj = datetime.datetime.now().time()
-
-        flattened = self.pickler.flatten(obj)
-        self.assertTrue(repr(obj) in flattened[tags.REPR])
-        self.assertTrue('datetime' in flattened[tags.REPR])
-
-        inflated = self.unpickler.restore(flattened)
-        self.assertEqual(obj, inflated)
-
-    def test_datetime_timedelta(self):
-        obj = datetime.timedelta(5)
-
-        flattened = self.pickler.flatten(obj)
-        self.assertTrue(repr(obj) in flattened[tags.REPR])
-        self.assertTrue('datetime' in flattened[tags.REPR])
-
-        inflated = self.unpickler.restore(flattened)
-        self.assertEqual(obj, inflated)
-
     def test_type_reference(self):
         """This test ensures that users can store references to types.
         """
@@ -412,7 +458,7 @@ class PicklingTestCase(unittest.TestCase):
 
         flattened = self.pickler.flatten(obj)
         self.assertEqual(flattened['classref'], {
-                            tags.TYPE: 'samples.Thing',
+                            tags.TYPE: 'jsonpickle._samples.Thing',
                          })
 
         inflated = self.unpickler.restore(flattened)
@@ -426,12 +472,30 @@ class PicklingTestCase(unittest.TestCase):
         inflated = self.unpickler.restore(flattened)
         self.assertEqual(obj, inflated)
 
+    def test_references(self):
+        obj_a = Thing('foo')
+        obj_b = Thing('bar')
+        coll = [obj_a, obj_b, obj_b]
+        flattened = self.pickler.flatten(coll)
+        inflated = self.unpickler.restore(flattened)
+        self.assertEqual(len(inflated), len(coll))
+        for x in range(len(coll)):
+            self.assertEqual(repr(coll[x]), repr(inflated[x]))
+
+    def test_list_subclass_with_init(self):
+        obj = ListSubclassWithInit('foo')
+        self.assertEqual(obj.attr, 'foo')
+        flattened = self.pickler.flatten(obj)
+        inflated = self.unpickler.restore(flattened)
+        self.assertEqual(type(inflated), ListSubclassWithInit)
+
 
 class JSONPickleTestCase(unittest.TestCase):
     def setUp(self):
         self.obj = Thing('A name')
-        self.expected_json = ('{"'+tags.OBJECT+'": "samples.Thing",'
-                              ' "name": "A name", "child": null}')
+        self.expected_json = (
+                '{"'+tags.OBJECT+'": "jsonpickle._samples.Thing",'
+                ' "name": "A name", "child": null}')
 
     def test_encode(self):
         pickled = jsonpickle.encode(self.obj)
@@ -460,39 +524,68 @@ class JSONPickleTestCase(unittest.TestCase):
         self.assertEqual(unpickled['é'.decode('utf-8')], 'é'.decode('utf-8'))
         self.assertTrue('é'.decode('utf-8') in unpickled)
 
-    def test_tuple_dict_keys(self):
-        """Test that we handle dictionaries with tuples as keys.
-        We do not model this presently, so ensure that we at
-        least convert those tuples to repr strings.
+    def test_tuple_dict_keys_default(self):
+        """Test that we handle dictionaries with tuples as keys."""
+        tuple_dict = {(1, 2): 3, (4, 5): { (7, 8): 9 }}
+        pickled = jsonpickle.encode(tuple_dict)
+        expect = {'(1, 2)': 3, '(4, 5)': {'(7, 8)': 9}}
+        actual = jsonpickle.decode(pickled)
+        self.assertEqual(expect, actual)
 
-        TODO: handle dictionaries with non-stringy keys.
-        """
-        pickled = jsonpickle.encode({(1, 2): 3,
-                                     (4, 5): { (7, 8): 9 }})
+        tuple_dict = {(1, 2): [1, 2]}
+        pickled = jsonpickle.encode(tuple_dict)
         unpickled = jsonpickle.decode(pickled)
-        subdict = unpickled['(4, 5)']
+        self.assertEqual(unpickled['(1, 2)'], [1, 2])
 
-        self.assertEqual(unpickled['(1, 2)'], 3)
-        self.assertEqual(subdict['(7, 8)'], 9)
+    def test_tuple_dict_keys_with_keys_enabled(self):
+        """Test that we handle dictionaries with tuples as keys."""
+        tuple_dict = {(1, 2): 3, (4, 5): { (7, 8): 9 }}
+        pickled = jsonpickle.encode(tuple_dict, keys=True)
+        expect = tuple_dict
+        actual = jsonpickle.decode(pickled, keys=True)
+        self.assertEqual(expect, actual)
 
-    def test_datetime_dict_keys(self):
-        """Test that we handle datetime objects as keys.
-        We do not model this presently, so ensure that we at
-        least convert those tuples into repr strings.
+        tuple_dict = {(1, 2): [1, 2]}
+        pickled = jsonpickle.encode(tuple_dict, keys=True)
+        unpickled = jsonpickle.decode(pickled, keys=True)
+        self.assertEqual(unpickled[(1, 2)], [1, 2])
 
-        """
-        pickled = jsonpickle.encode({datetime.datetime(2008, 12, 31): True})
-        unpickled = jsonpickle.decode(pickled)
-        self.assertTrue(unpickled['datetime.datetime(2008, 12, 31, 0, 0)'])
+    def test_datetime_dict_keys_defaults(self):
+        """Test that we handle datetime objects as keys."""
+        datetime_dict = {datetime.datetime(2008, 12, 31): True}
+        pickled = jsonpickle.encode(datetime_dict)
+        expect = {'datetime.datetime(2008, 12, 31, 0, 0)': True}
+        actual = jsonpickle.decode(pickled)
+        self.assertEqual(expect, actual)
+
+    def test_datetime_dict_keys_with_keys_enabled(self):
+        """Test that we handle datetime objects as keys."""
+        datetime_dict = {datetime.datetime(2008, 12, 31): True}
+        pickled = jsonpickle.encode(datetime_dict, keys=True)
+        expect = datetime_dict
+        actual = jsonpickle.decode(pickled, keys=True)
+        self.assertEqual(expect, actual)
 
     def test_object_dict_keys(self):
         """Test that we handle random objects as keys.
 
         """
-        pickled = jsonpickle.encode({Thing('random'): True})
+        thing = Thing('random')
+        pickled = jsonpickle.encode({thing: True})
         unpickled = jsonpickle.decode(pickled)
-        self.assertEqual(unpickled,
-                         {u'samples.Thing("random")': True})
+        self.assertEqual(unpickled, {unicode('Thing("random")'): True})
+
+    def test_int_dict_keys_defaults(self):
+        int_dict = {1000: [1, 2]}
+        pickled = jsonpickle.encode(int_dict)
+        unpickled = jsonpickle.decode(pickled)
+        self.assertEqual(unpickled['1000'], [1, 2])
+
+    def test_int_dict_keys_with_keys_enabled(self):
+        int_dict = {1000: [1, 2]}
+        pickled = jsonpickle.encode(int_dict, keys=True)
+        unpickled = jsonpickle.decode(pickled, keys=True)
+        self.assertEqual(unpickled[1000], [1, 2])
 
     def test_list_of_objects(self):
         """Test that objects in lists are referenced correctly"""
@@ -544,13 +637,13 @@ class JSONPickleTestCase(unittest.TestCase):
                 backend in jsonpickle.json._encoder_options or
                 backend in jsonpickle.json._decoder_exceptions)
 
-    def test_load_backend_skips_bad_inputs(self):
+    def test_load_backend_skips_bad_encode(self):
         """Test that we ignore bad encoders"""
 
         jsonpickle.load_backend('os.path', 'bad!', 'split', AttributeError)
         self.failIf(self._backend_is_partially_loaded('os.path'))
 
-    def test_load_backend_skips_bad_inputs(self):
+    def test_load_backend_skips_bad_decode(self):
         """Test that we ignore bad decoders"""
 
         jsonpickle.load_backend('os.path', 'join', 'bad!', AttributeError)
@@ -562,6 +655,82 @@ class JSONPickleTestCase(unittest.TestCase):
         jsonpickle.load_backend('os.path', 'join', 'split', 'bad!')
         self.failIf(self._backend_is_partially_loaded('os.path'))
 
+    def test_list_item_reference(self):
+        thing = Thing('parent')
+        thing.child = Thing('child')
+        thing.child.refs = [thing]
+
+        encoded = jsonpickle.encode(thing)
+        decoded = jsonpickle.decode(encoded)
+
+        self.assertEqual(id(decoded.child.refs[0]), id(decoded))
+
+    def test_reference_to_list(self):
+        thing = Thing('parent')
+        thing.a = [1]
+        thing.b = thing.a
+        thing.b.append(thing.a)
+        thing.b.append([thing.a])
+
+        encoded = jsonpickle.encode(thing)
+        decoded = jsonpickle.decode(encoded)
+
+        self.assertEqual(decoded.a[0], 1)
+        self.assertEqual(decoded.b[0], 1)
+        self.assertEqual(id(decoded.a), id(decoded.b))
+        self.assertEqual(id(decoded.a), id(decoded.a[1]))
+        self.assertEqual(id(decoded.a), id(decoded.a[2][0]))
+
+    def test_repr_using_jsonpickle(self):
+        thing = ObjWithJsonPickleRepr()
+        thing.child = ObjWithJsonPickleRepr()
+        thing.child.parent = thing
+
+        encoded = jsonpickle.encode(thing)
+        decoded = jsonpickle.decode(encoded)
+
+        self.assertEqual(id(decoded), id(decoded.child.parent))
+
+    def test_ordered_dict(self):
+        if sys.version_info < (2, 7):
+            return
+
+        d = collections.OrderedDict()
+        d.update(c=3)
+        d.update(a=1)
+        d.update(b=2)
+
+        encoded = jsonpickle.encode(d)
+        decoded = jsonpickle.decode(encoded)
+
+        self.assertEqual(d, decoded)
+
+    def test_make_refs_disabled_list(self):
+        obj_a = Thing('foo')
+        obj_b = Thing('bar')
+        coll = [obj_a, obj_b, obj_b]
+        encoded = jsonpickle.encode(coll, make_refs=False)
+        decoded = jsonpickle.decode(encoded)
+
+        self.assertEqual(len(decoded), 3)
+        self.assertTrue(decoded[0] is not decoded[1])
+        self.assertTrue(decoded[1] is not decoded[2])
+
+    def test_make_refs_disabled_reference_to_list(self):
+        thing = Thing('parent')
+        thing.a = [1]
+        thing.b = thing.a
+        thing.b.append(thing.a)
+        thing.b.append([thing.a])
+
+        encoded = jsonpickle.encode(thing, make_refs=False)
+        decoded = jsonpickle.decode(encoded)
+
+        self.assertEqual(decoded.a[0], 1)
+        self.assertEqual(decoded.b[0:3], '[1,')
+        self.assertEqual(decoded.a[1][0:3], '[1,')
+        self.assertEqual(decoded.a[2][0][0:3], '[1,')
+
 
 # Test classes for ExternalHandlerTestCase
 class Mixin(object):
@@ -576,6 +745,7 @@ class UnicodeMixin(unicode, Mixin):
 
 
 class UnicodeMixinHandler(handlers.BaseHandler):
+    _handles = UnicodeMixin,
     def flatten(self, obj, data):
         data['value'] = obj
         return data
@@ -585,15 +755,10 @@ class UnicodeMixinHandler(handlers.BaseHandler):
 
 
 class ExternalHandlerTestCase(unittest.TestCase):
-    def setUp(self):
-        handlers.registry.register(UnicodeMixin, UnicodeMixinHandler)
-
-    def tearDown(self):
-        handlers.registry.unregister(UnicodeMixin)
-
     def test_unicode_mixin(self):
         obj = UnicodeMixin('test')
-        self.assertEqual(unicode(obj), u'test')
+        self.assertEqual(type(obj), UnicodeMixin)
+        self.assertEqual(unicode(obj), unicode('test'))
 
         # Encode into JSON
         content = jsonpickle.encode(obj)
@@ -602,7 +767,7 @@ class ExternalHandlerTestCase(unittest.TestCase):
         new_obj = jsonpickle.decode(content)
         new_obj += ' passed'
 
-        self.assertEqual(unicode(new_obj), u'test passed')
+        self.assertEqual(unicode(new_obj), unicode('test passed'))
         self.assertEqual(type(new_obj), UnicodeMixin)
         self.assertTrue(new_obj.ok())
 
